@@ -1,53 +1,132 @@
 # File contains helper functions written in R for PanelMatch functionality
 perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.method, size.match,
                                ordered.data, match.missing, covs.formula, verbose,
-                               mset.object = NULL, lead, outcome.var = NULL, forbid.treatment.reversal = FALSE, qoi = "",
-                               matching = TRUE, exact.matching.variables = NULL, listwise.deletion,
-                               use.diag.covmat = FALSE)
+                               mset.object = NULL, lead, outcome.var = NULL,
+                               forbid.treatment.reversal = FALSE, qoi = "",
+                               matching = TRUE, exact.matching.variables = NULL,
+                               listwise.deletion,
+                               use.diag.covmat = FALSE, caliper.formula = NULL,
+                               continuous.treatment.info = NULL,
+                               network.caliper.info = NULL,
+                               network.refinement.info = NULL,
+                               adjacency.matrix,
+                               neighborhood.degree,
+                               placebo.test = FALSE)
 {
-  if(!is.null(mset.object))
+
+  if ( !is.null(mset.object) ) stop('This should never run!')
+  if (identical(class(ordered.data[, unit.id]), "numeric"))
   {
-    lag = attr(mset.object, "lag")
-    time.id = attr(mset.object, "t.var")
-    unit.id = attr(mset.object, "id.var")
-    treatment = attr(mset.object, "treatment.var")
-    refinement.method = attr(mset.object, "refinement.method")
-    size.match = attr(mset.object, "max.match.size")
-    covs.formula = attr(mset.object, "covs.formula")
-    match.missing <- attr(mset.object, "match.missing")
-    verbose = FALSE
-    msets <- mset.object
+    warning("converting unit id variable data to integer")
+    class(ordered.data[, unit.id]) <- "integer"
   }
-  else
+  continuous.treatment <- !is.null(continuous.treatment.info) #to make this easier
+  if (continuous.treatment)
   {
 
-    temp.treateds <- findAllTreated(ordered.data, treatedvar = treatment, time.var = time.id,
-                                    unit.var = unit.id, hasbeensorted = TRUE)
-    idx <- !((temp.treateds[, time.id] - lag) < min(ordered.data[, time.id]))
-    temp.treateds <- temp.treateds[idx, ]
-    if(nrow(temp.treateds) == 0)
+    temp.treateds <- findContinuousTreated(dmat = ordered.data, treatedvar = treatment, time.var = time.id,
+                          unit.var = unit.id, qoi = qoi,
+                          continuous.treatment.info = continuous.treatment.info)
+
+    if(!is.null(continuous.treatment.info[["minimum.treatment.value"]]))
     {
-      warn.str <- paste0("no viable treated units for ", qoi, " specification")
-      stop(warn.str)
+      indx <- temp.treateds[, treatment] >= continuous.treatment.info[["minimum.treatment.value"]]
+      temp.treateds <- temp.treateds[indx,]
+    }
+    if(!is.null(continuous.treatment.info[["maximum.treatment.value"]]))
+    {
+      indx <- temp.treateds[, treatment] <= continuous.treatment.info[["maximum.treatment.value"]]
+      temp.treateds <- temp.treateds[indx,]
+    }
+    ## add filter in here
+  } else
+  {
+    temp.treateds <- findBinaryTreated(ordered.data, treatedvar = treatment, time.var = time.id,
+                                       unit.var = unit.id, hasbeensorted = TRUE)
+  }
+
+
+
+  idx <- !((temp.treateds[, time.id] - lag) < min(ordered.data[, time.id]))
+  temp.treateds <- temp.treateds[idx, ]
+  if (nrow(temp.treateds) == 0)
+  {
+    warn.str <- paste0("no viable treated units for ", qoi, " specification")
+    stop(warn.str)
+  }
+  msets <- get.matchedsets(temp.treateds[, time.id], temp.treateds[, unit.id], data = ordered.data,
+                           L = lag, t.column = time.id, id.column = unit.id,
+                           treatedvar = treatment, hasbeensorted = TRUE,
+                           match.on.missingness = match.missing, matching = TRUE,
+                           continuous = continuous.treatment,
+                           continuous.treatment.info = continuous.treatment.info)
+  e.sets <- msets[sapply(msets, length) == 0]
+  msets <- msets[sapply(msets, length) > 0 ]
+  if (length(msets) == 0)
+  {
+    t.attributes <- attributes(e.sets)[names(attributes(e.sets)) != "names"]
+    msets <- e.sets
+    for (idx in names(t.attributes))
+    {
+      attr(msets, idx) <- t.attributes[[idx]]
+    }
+    attr(msets, "covs.formula") <- covs.formula
+    attr(msets, "match.missing") <- match.missing
+    return(msets)
+    return(msets)
+  }
+
+  msets <- clean_leads(msets, ordered.data, max(lead), time.id, unit.id, outcome.var)
+
+
+  #browser()
+  if (placebo.test)
+  {
+    treated.ts <- as.integer(sub(".*\\.", "", names(msets)))
+    treated.ids <- as.integer(sub("\\..*", "", names(msets)))
+
+    rownames(ordered.data) <- paste0(ordered.data[, unit.id],".", ordered.data[, time.id])
+
+    for (i in 1:length(msets)) {
+      cur.id <- treated.ids[i]
+      cur.t <- treated.ts[i]
+      to.check <- paste0(cur.id, ".", (cur.t - lag):(cur.t - 1))
+
+      if ( any(is.na(ordered.data[to.check, outcome.var])) )
+      {
+
+        names(msets)[i] <- "remove"
+      } else
+      {
+        for (j in 1:length(msets[[i]])) {
+          to.check <- paste0(msets[[i]][j], ".", (cur.t - lag):(cur.t - 1))
+          if ( any(is.na(ordered.data[to.check, outcome.var])) )
+          {
+            msets[[i]][j] <- NA
+          }
+        }
+      }
+    }
+    
+    msets <- msets[names(msets) != "remove"]
+
+    for (i in 1:length(msets)) {
+      msets[[i]] <- msets[[i]][!is.na(msets[[i]])]
     }
-    msets <- get.matchedsets(temp.treateds[, time.id], temp.treateds[, unit.id], ordered.data,
-                             lag, time.id, unit.id, treatment, hasbeensorted = TRUE, match.on.missingness = match.missing, matching)
-    e.sets <- msets[sapply(msets, length) == 0]
     msets <- msets[sapply(msets, length) > 0 ]
-
-    msets <- clean_leads(msets, ordered.data, max(lead), time.id, unit.id, outcome.var)
-
-    if(forbid.treatment.reversal)
-    {
-      msets <- enforce_lead_restrictions(msets, ordered.data, max(lead), time.id, unit.id, treatment.var = treatment)
-    }
-    if(length(msets) == 0)
-    {
-      warn.str <- paste0("no matched sets for ", qoi, " specification")
-      stop(warn.str)
-    }
   }
-  if(!is.null(exact.matching.variables))
+
+  if (forbid.treatment.reversal)
+  {
+    msets <- enforce_lead_restrictions(msets, ordered.data, max(lead), time.id, unit.id, treatment.var = treatment)
+  }
+  if (length(msets) == 0)
+  {
+    warn.str <- paste0("no matched sets for ", qoi, " specification")
+    stop(warn.str)
+  }
+
+  if (!is.null(exact.matching.variables))
   {
     msets <- do_exact_matching(msets, ordered.data, exact.matching.variables)
 
@@ -55,6 +134,59 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
     msets <- msets[sapply(msets, length) > 0 ]
   }
 
+
+
+  ####apply calipers here
+  if (!is.null(caliper.formula))
+  {
+    msets <- handle_calipers(plain.ordered.data = ordered.data, caliper.formula,
+                             matched.sets = msets, lag.window = 0:lag)
+
+  }
+
+  ####CALCULATE INFO ABOUT DIRECTIONAL CHANGE FOR CONTINUOUS TREATMENT HERE#####
+
+  # always calculate the treatment change, not just for continuous treatment cases
+
+  if (length(msets) > 0)
+  {
+    msets <- identifyDirectionalChanges(msets, ordered.data,
+                                        unit.id, time.id, treatment, qoi)
+  }
+  if (length(e.sets) > 0)
+  {
+    e.sets <- identifyDirectionalChanges(e.sets, ordered.data,
+                                         unit.id, time.id, treatment, qoi)
+  }
+
+
+  if (!is.null(continuous.treatment.info))
+  {
+    if (continuous.treatment.info[["direction"]] == "positive")
+    {
+      idx <- sapply(msets, function(x) attr(x, "treatment.change")) >= 0
+      msets <- msets[idx]
+      if (length(e.sets) > 0)
+      {
+        idx <- sapply(e.sets, function(x) attr(x, "treatment.change")) >= 0
+        e.sets <- e.sets[idx]
+      }
+
+    } else if (continuous.treatment.info[["direction"]] == "negative")
+    {
+      idx <- sapply(msets, function(x) attr(x, "treatment.change")) <= 0
+      msets <- msets[idx]
+      if (length(e.sets) > 0)
+      {
+        idx <- sapply(e.sets, function(x) attr(x, "treatment.change")) >= 0
+        e.sets <- e.sets[idx]
+      }
+    } else
+    {
+      stop("direction not well specified")
+    }
+  }
+
   if(refinement.method == "none")
   {
     for(i in 1:length(msets))
@@ -71,13 +203,36 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
     }
     attr(msets, "covs.formula") <- covs.formula
     attr(msets, "match.missing") <- match.missing
+
     return(msets)
   }
 
-  treated.ts <- as.numeric(unlist(strsplit(names(msets), split = "[.]"))[c(F,T)])
-  treated.ids <- as.numeric(unlist(strsplit(names(msets), split = "[.]"))[c(T,F)])
+  treated.ts <- as.integer(sub(".*\\.", "", names(msets)))
+  treated.ids <- as.integer(sub("\\..*", "", names(msets)))
+
+
+  # ###################################network code (old?)########################################
+  # if (!is.null(network.caliper.info) || !is.null(network.refinement.info))
+  # {
+  #   # browser()
+  #   treated.names <- c(names(msets), names(e.sets))
+  #   ordered.data <- calculate_neighbor_treatment(ordered.data, adjacency.matrix,
+  #                                                neighborhood.degree, unit.id,
+  #                                                time.id, treatment)
+  #   ll <- handle_network_caliper_and_refinement(network.caliper.info, network.refinement.info, ordered.data,
+  #                                               adjacency.matrix, neighborhood.degree,
+  #                                               unit.id, time.id, treatment,
+  #                                               covs.formula, caliper.formula)
+  #   covs.formula <- ll[[1]]
+  #   caliper.formula <- ll[[2]]
+  # }
+  # ###########################################################################
 
   ordered.data <- parse_and_prep(formula = covs.formula, data = ordered.data)
+  if (any(c("character", "factor") %in% sapply(ordered.data, class)))
+  {
+    stop("please convert covs.formula variables to numerical data")
+  }
   if(any(apply(ordered.data, 2, FUN = function(x) any(is.infinite(x)))))
   {
     stop("Data needed for refinement contains infinite values. Code cannot proceed!")
@@ -85,7 +240,6 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
   ################################################################################################
   if(listwise.deletion) #code will just return from here when listwise.deletion = T
   {
-
     msets <- lwd_refinement(msets, ordered.data, treated.ts, treated.ids, lag,
                             time.id, unit.id, lead, refinement.method, treatment, size.match,
                             match.missing, covs.formula, verbose, outcome.var, e.sets,
@@ -99,13 +253,20 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
   }
 
 
-  if(refinement.method == "mahalanobis")
+  if (refinement.method == "mahalanobis")
   {
+
+    old.lag <- lag
+    lag <- 0
     tlist <- expand.treated.ts(lag, treated.ts = treated.ts)
-    idxlist <- get_yearly_dmats(ordered.data, treated.ids, tlist, paste0(ordered.data[,unit.id], ".",
-                                                                         ordered.data[, time.id]), matched_sets = msets, lag)
+
+    idxlist <- get_yearly_dmats(ordered.data, treated.ids, tlist, matched_sets = msets, lag)
+
     mahalmats <- build_maha_mats(ordered_expanded_data = ordered.data, idx =  idxlist)
+
     msets <- handle_mahalanobis_calculations(mahalmats, msets, size.match, verbose, use.diagonal.covmat = use.diag.covmat)
+
+    lag <- old.lag
   }
   if(refinement.method == "ps.msm.weight" | refinement.method == "CBPS.msm.weight")
   {
@@ -114,8 +275,7 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
     {
         f <- lead[i]
         tf <- expand.treated.ts(lag, treated.ts = treated.ts + f)
-        tf.index <- get_yearly_dmats(ordered.data, treated.ids, tf, paste0(ordered.data[,unit.id], ".",
-                                                                           ordered.data[, time.id]), matched_sets = msets, lag)
+        tf.index <- get_yearly_dmats(ordered.data, treated.ids, tf, matched_sets = msets, lag)
         expanded.sets.tf <- build_ps_data(tf.index, ordered.data, lag)
         #pre.pooled <- ordered.data[ordered.data[, time.id] %in% (treated.ts + f), ]
         pre.pooled <- rbindlist(expanded.sets.tf)
@@ -124,6 +284,7 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
         #do the column removal thing
         cols.to.remove <- which(unlist(lapply(pooled, function(x){all(x[1] == x)}))) #checking for columns that only have one value
         cols.to.remove <- unique(c(cols.to.remove, which(!colnames(pooled) %in% colnames(t(unique(t(pooled))))))) #removing columns that are identical to another column
+        cols.to.remove <- cols.to.remove[cols.to.remove > 3] #leave the first three columns alone
         if(length(cols.to.remove) > 0)
         {
           class(pooled) <- c("data.frame")
@@ -151,7 +312,7 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
           #}
 
         }
-        if(refinement.method == "CBPS.msm.weight") #obviously update these conditionals
+        if (refinement.method == "CBPS.msm.weight") #obviously update these conditionals
         {
           dummy <- capture.output(fit.tf <- (CBPS::CBPS(reformulate(response = treatment, termlabels = colnames(pooled)[-c(1:3)]),
                                                 family = binomial(link = "logit"), data = pooled)))
@@ -171,16 +332,17 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
   }  #not msm
   if(all(refinement.method %in% c("CBPS.weight", "CBPS.match", "ps.weight", "ps.match")))
   {
+
     if(!all(refinement.method %in% c("CBPS.weight", "CBPS.match", "ps.weight", "ps.match"))) stop("please choose valid refinement method")
     tlist <- expand.treated.ts(lag, treated.ts = treated.ts)
-    idxlist <- get_yearly_dmats(ordered.data, treated.ids, tlist, paste0(ordered.data[,unit.id], ".",
-                                                                         ordered.data[, time.id]), matched_sets = msets, lag)
+    idxlist <- get_yearly_dmats(ordered.data, treated.ids, tlist, matched_sets = msets, lag)
     expanded.sets.t0 <- build_ps_data(idxlist, ordered.data, lag)
     pre.pooled <- rbindlist(expanded.sets.t0)
     pooled <- unique(pre.pooled[complete.cases(pre.pooled), ])
 
     cols.to.remove <- which(unlist(lapply(pooled, function(x){all(x[1] == x)}))) #checking for columns that only have one value
     cols.to.remove <- unique(c(cols.to.remove, which(!colnames(pooled) %in% colnames(t(unique(t(pooled))))))) #removing columns that are identical to another column
+    cols.to.remove <- cols.to.remove[cols.to.remove > 3] #leave the first three columns alone
     if(length(cols.to.remove) > 0)
     {
       class(pooled) <- c("data.frame")
@@ -194,24 +356,14 @@ perform_refinement <- function(lag, time.id, unit.id, treatment, refinement.meth
     if(qr(pooled)$rank != ncol(pooled))
     {
 
-      # print("Data used to generate propensity scores is not linearly independent. Calculations cannot be completed.
-      #       Would you like to save the problematic matrix to file for manual inspection? File and variable will be saved as 'problematic_matrix.rda'. ")
-      # inkey <- readline("Press 'y' to save and any other key to do nothing: ")
-      # if(inkey == "y")
-      # {
-      #   problematic_matrix <- pooled
-      #   save(problematic_matrix, file = "problematic_matrix.rda")
-      #   stop("PanelMatch terminated")
-      # }
-      # else
-      # {
+
       stop("Error: Provided data is not linearly independent so calculations cannot be completed. Please check the data set for any redundant, unnecessary, or problematic information.")
-      #}
+
 
     }
     if(refinement.method == "CBPS.weight" | refinement.method == "CBPS.match")
     {
-      dummy <-capture.output(fit0 <- (CBPS::CBPS(reformulate(response = treatment, termlabels = colnames(pooled)[-c(1:3)]),
+      dummy <- capture.output(fit0 <- (CBPS::CBPS(reformulate(response = treatment, termlabels = colnames(pooled)[-c(1:3)]),
                                           family = binomial(link = "logit"), data = pooled)))
     }
     if(refinement.method == "ps.weight" | refinement.method == "ps.match")
@@ -258,13 +410,16 @@ parse_and_prep <- function(formula, data)
 
   lag <- function(y, lwindow)
   {
+
     sapply(lwindow, internal.lag, x = y)
   }
 
+
   apply_formula <- function(x, form)
   {
     attr(form, ".Environment") <- environment()
     tdf <- model.frame(form, x, na.action = NULL)
+
     cbind(x[, c(1, 2, 3)], model.matrix(form, tdf)[, -1])
   }
 
@@ -303,6 +458,73 @@ build_maha_mats <- function(idx, ordered_expanded_data)
 }
 
 
+#modified version of build mahah mats but with refinement handled immediately
+handle_distance_matrices <- function(ordered_expanded_data, matched.sets, calipervalue,
+                                     calipermethod, isfactor, use.sd, id.var,
+                                     time.var, lag.in, continuous.matching = FALSE,
+                                     control.threshold = NULL)
+{
+
+  unnest <- function(matched.set, treated.unit.info,
+                     lag.in_, ordered_expanded_data_, is.continuous.matching = FALSE,
+                     idvar, timevar, all.treated.info, control.threshold = NULL)
+  {
+    treated.ts <- as.integer(sub(".*\\.", "", treated.unit.info))
+    treated.ids <- as.integer(sub("\\..*", "", treated.unit.info))
+
+    all.treated.ts <- as.integer(sub(".*\\.", "", all.treated.info))
+    all.treated.ids <- as.integer(sub("\\..*", "", all.treated.info))
+    if (is.continuous.matching)
+    {
+
+      matched.set <- prepContinuousControlUnits(ordered_expanded_data_, idvar,
+                                                time.var, all.treated.ids,
+                                                all.treated.ts, treated.ids,
+                                                treated.ts, control.threshold)
+      #full.controls <- unique(ordered_expanded_data_[, idvar])
+      ####THIS IS WHERE WE WANT TO UPDATE TO APPLY A FILTER TO CONTROL UNITS
+      #browser()
+      #not.valid.ids <- all.treated.ids[all.treated.ts %in% treated.ts] #cant include other treated units from the same time
+      #matched.set <- full.controls[!full.controls %in% not.valid.ids] # start with everything, then remove any units that are treated
+      # during the same period as the current t/id pair under consideration
+    }
+    tlist <- expand.treated.ts(lag.in_, treated.ts = treated.ts)
+
+    idxlist <- get_yearly_dmats(ordered_expanded_data_, treated.ids, tlist,
+                                matched_sets = list(matched.set), lag.in_)
+    rr <- lapply(unlist(idxlist, recursive = FALSE), function(x) {ordered_expanded_data_[x, ]})
+
+
+
+    tset <- handle_perlag_caliper_calculations(rr, matched.set, calipermethod, calipervalue,
+                                        isfactor, use.sd,
+                                        ordered_expanded_data,
+                                        id.var, time.var, names(matched.sets)) #sloppy style, fix later
+    return(tset)
+  }
+
+
+  result <- mapply(FUN = unnest, matched.set = matched.sets, treated.unit.info = names(matched.sets),
+                   MoreArgs = list(lag.in_ = lag.in,
+                                   ordered_expanded_data_ = ordered_expanded_data,
+                                   is.continuous.matching = continuous.matching,
+                                   idvar = id.var,
+                                   timevar = time.var,
+                                   all.treated.info = names(matched.sets),
+                                   control.threshold = control.threshold),
+                   SIMPLIFY = FALSE)
+  #result <- mapply(FUN = unnest, mset.idx = idx, matched.set = matched.sets, SIMPLIFY = FALSE)
+  names(result) <- names(matched.sets)
+  if (!continuous.matching)
+  {
+    result <- result[sapply(result, length) > 0]
+  }
+
+  return(result)
+
+
+}
+
 #use col.index to determine which columns we want to "scan" for missing data
 # Note that in earlier points in the code, we rearrange the columns and prepare the data frame such that cols 1-4 are bookkeeping (unit id, time id, treated variable, unlagged outcome variable)
 # and all remaining columns are used in the calculations after going through parse_and_prep function, so col.index should usually be 5:ncol(data)
@@ -369,7 +591,6 @@ find_ps <- function(sets, fitted.model)
 # These functions could use some work to be better optimized. For instance, when the "verbose" argument is set to true, they will essentially do all of the refinement calculations twice.
 handle_mahalanobis_calculations <- function(mahal.nested.list, msets, max.size, verbose, use.diagonal.covmat)
 {
-
   do.calcs <- function(year.df)
   {
     if(nrow(year.df) == 2)
@@ -422,23 +643,28 @@ handle_mahalanobis_calculations <- function(mahal.nested.list, msets, max.size,
   }
   handle_set <- function(sub.list, max.set.size, idx)
   {
+
     results.temp <- lapply(sub.list, do.calcs)
     tmat <- do.call(rbind, results.temp)
     colnames(tmat) <- NULL
     dists <- colMeans(tmat)
-    n.dists <- dists[dists > 0]
-    if(length(n.dists) == 0)
-    {
-      w <- 1 / length(dists)
-      newdists <- dists
-      newdists <- rep(w, length(newdists))
-    }
+    #n.dists <- dists[dists > 0]
+    n.dists <- dists
+    #if(length(n.dists) == 0)
+    #{
+    #  w <- 1 / length(dists)
+    #  newdists <- dists
+    #  newdists <- rep(w, length(newdists))
+    #}
     #if(length(n.dists) == 0 ) browser()#stop("a matched set contain only identical units. Please examine the data and remove this set.")
-    else if(length(n.dists) < max.set.size)
+    #else
+
+    if(length(n.dists) < max.set.size) #case where total number of units in matched set < max.set size
     {
       w <- 1 / length(n.dists)
       newdists <- dists
-      newdists[newdists > 0 ] <- w
+      newdists <- rep(w, length(newdists))
+      #newdists[newdists > 0 ] <- w
     }
     else
     {
@@ -446,14 +672,24 @@ handle_mahalanobis_calculations <- function(mahal.nested.list, msets, max.size,
       scoretobeat <- max(utils::head(ordered.dists, n = max.set.size + 1))
       # might have situation where the Mth largest distance is the same as the Mth - 1 distance. This means that we either choose to leave out both and have a matched set smaller than the max,
       # or include both of them and relax the size of our maximum set size
-      if(sum(dists < scoretobeat & dists > 0) < max.set.size) #change this if we want to be more strict about max.set.size enforcements
+      # if(sum(dists < scoretobeat & dists > 0) < max.set.size) #change this if we want to be more strict about max.set.size enforcements
+      # {
+      #   new.denom <- sum(dists <= scoretobeat & dists > 0)
+      #   newdists <- ifelse(dists <= scoretobeat & dists > 0, 1 / new.denom, 0)
+      # }
+      # else
+      # {
+      #   newdists <- ifelse(dists < scoretobeat & dists > 0, 1 / max.set.size, 0)
+      # }
+
+      if(sum(dists < scoretobeat) < max.set.size) #change this if we want to be more strict about max.set.size enforcements
       {
-        new.denom <- sum(dists <= scoretobeat & dists > 0)
-        newdists <- ifelse(dists <= scoretobeat & dists > 0, 1 / new.denom, 0)
+        new.denom <- sum(dists <= scoretobeat)
+        newdists <- ifelse(dists <= scoretobeat, 1 / new.denom, 0)
       }
       else
       {
-        newdists <- ifelse(dists < scoretobeat & dists > 0, 1 / max.set.size, 0)
+        newdists <- ifelse(dists < scoretobeat, 1 / max.set.size, 0)
       }
 
     }
@@ -462,7 +698,9 @@ handle_mahalanobis_calculations <- function(mahal.nested.list, msets, max.size,
 
   }
 
-  scores <- mapply(FUN = handle_set, sub.list = mahal.nested.list, idx = 1:length(msets), MoreArgs = list(max.set.size = max.size), SIMPLIFY = FALSE)
+  scores <- mapply(FUN = handle_set, sub.list = mahal.nested.list,
+                   idx = 1:length(msets),
+                   MoreArgs = list(max.set.size = max.size), SIMPLIFY = FALSE)
   for(i in 1:length(msets))
   {
     names(scores[[i]]) <- msets[[i]]
@@ -508,7 +746,7 @@ handle_ps_weighted <- function(just.ps.sets, msets, refinement.method)
     }
     else
     {
-      wts <- ( vec.ratio ) / sum( vec.ratio )  
+      wts <- ( vec.ratio ) / sum( vec.ratio )
     }
     return(as.vector(wts))
   }
@@ -522,7 +760,9 @@ handle_ps_weighted <- function(just.ps.sets, msets, refinement.method)
   return(msets)
 }
 
-handle_ps_match <- function(just.ps.sets, msets, refinement.method, verbose, max.set.size)
+handle_ps_match <- function(just.ps.sets, msets,
+                            refinement.method,
+                            verbose, max.set.size)
 {
   handle_set <- function(set, max.size)
   {
@@ -592,14 +832,19 @@ clean_leads <- function(matched_sets, ordered.data, max.lead, t.var, id.var, out
 {
   #CHECK TO MAKE SURE COLUMNS ARE IN ORDER
 
-  ordered.data <- ordered.data[order(ordered.data[,id.var], ordered.data[,t.var]), ]
-  compmat <- data.table::dcast(data.table::as.data.table(ordered.data), formula = paste0(id.var, "~", t.var), value.var = outcome.var)
-  ts <- as.numeric(unlist(strsplit(names(matched_sets), split = "[.]"))[c(F,T)])
-  tids <- as.numeric(unlist(strsplit(names(matched_sets), split = "[.]"))[c(T,F)])
+  old.attributes <- attributes(matched_sets)[names(attributes(matched_sets)) != "names"]
+
+  ts <- as.numeric(sub(".*\\.", "", names(matched_sets)))
+  tids <- as.numeric(sub("\\..*", "", names(matched_sets)))
   class(matched_sets) <- "list" #so that Rcpp::List is accurate when we pass it into cpp functions
-  compmat <- data.matrix(compmat)
+
+
+  idx <- check_missing_data_treated_units(subset_data = as.matrix(ordered.data[, c(id.var,t.var,outcome.var)]),
+                                           sets = matched_sets,
+                                          tid_pairs = paste0(ordered.data[, id.var], ".", ordered.data[, t.var]),
+                                          treated_tid_pairs = names(matched_sets),
+                                           treated_ids = tids, lead =  max.lead)
 
-  idx <- check_treated_units(compmat = compmat, compmat_row_units = as.numeric(compmat[, 1]), compmat_cols = as.numeric(colnames(compmat)[2:ncol(compmat)]), lead = max.lead, treated_ids = tids, treated_ts = ts)
   if(all(!idx)) stop("estimation not possible: All treated units are missing data necessary for the calculations to proceed")
   if(any(!idx))
   {
@@ -610,40 +855,48 @@ clean_leads <- function(matched_sets, ordered.data, max.lead, t.var, id.var, out
   }
   #colnames must be numeric in some form because we must be able to sort them into an ascending column order
   class(matched_sets) <- "list" # for rcpp reasons again
-  ll <- re_norm_index(compmat = compmat, compmat_row_units = as.numeric(compmat[, 1]), compmat_cols = as.numeric(colnames(compmat)[2:ncol(compmat)]), lead = max.lead, sets = matched_sets, control_start_years = ts)
-  idx <- needs_renormalization(ll)
-  class(matched_sets) <- c("matched.set", "list")
-  if(any(idx))
+
+  if(any(idx)) #bit of a trivial condition, if there are any treated units with matched sets left
   {
-    sub.index <- ll[idx]
-    sub.set <- matched_sets[idx]
+    create_control_maps <- function(matched_set, time)
+    {
+      return(paste0(matched_set, ".", time))
+    }
+
+    prepped_sets <- mapply(create_control_maps, matched_set = matched_sets, time = ts, SIMPLIFY = FALSE)
+
+
+
+    tpx <- check_missing_data_control_units(subset_data = as.matrix(ordered.data[, c(id.var,t.var,outcome.var)]),
+                                            sets = matched_sets,
+                                            prepared_sets = prepped_sets,
+                                            tid_pairs = paste0(ordered.data[, id.var], ".", ordered.data[, t.var]),
+                                            lead =  max.lead)
+
     create_new_sets <- function(set, index)
     {
       return(set[index])
     }
-    sub.set.new <- mapply(FUN = create_new_sets, sub.set, sub.index, SIMPLIFY = FALSE)
-    attributes(sub.set.new) <- attributes(sub.set)
-    all.gone.counter <- sapply(sub.set.new, function(x){sum(x)})
-    if(sum(all.gone.counter == 0) > 0) #case in which all the controls in a particular group were dropped
-    {
-      #warning("all controls in a particular matched set were removed due to missing data")
+    sub_sets <- mapply(FUN = create_new_sets, matched_sets, tpx, SIMPLIFY = FALSE)
 
-      idx[all.gone.counter == 0] <- FALSE
-      sub.index <- ll[idx]
-      sub.set <- matched_sets[idx]
-      create_new_sets <- function(set, index)
-      {
-        return(set[index])
-      }
-      sub.set.new <- mapply(FUN = create_new_sets, sub.set, sub.index, SIMPLIFY = FALSE)
-      attributes(sub.set.new) <- attributes(sub.set)
+    if(all(sapply(sub_sets, length) == 0)) stop('estimation not possible: none of the matched sets have viable control units due to a lack of necessary data')
+
+    matched_sets <- sub_sets[sapply(sub_sets, length) > 0]
+
+    for(idx in names(old.attributes))
+    {
+
+      attr(matched_sets, idx) <- old.attributes[[idx]]
     }
-    if(all(sapply(sub.set.new, length) == 0)) stop('estimation not possible: none of the matched sets have viable control units due to a lack of necessary data')
-    #pm2 <- perform_refinement(ordered.data = ordered.data, mset.object = sub.set.new)
-    #matched_sets[idx] <- pm2
-    matched_sets[idx] <- sub.set.new
-    matched_sets <- matched_sets[sapply(matched_sets, length) > 0]
+
   }
+  class(matched_sets) <- c("matched.set")
+
+  return(matched_sets)
+
+
+
+
   return(matched_sets)
 
 }
@@ -652,19 +905,19 @@ clean_leads <- function(matched_sets, ordered.data, max.lead, t.var, id.var, out
 enforce_lead_restrictions <- function(matched_sets, ordered.data, max.lead, t.var, id.var, treatment.var)
 {
   #CHECK TO MAKE SURE COLUMNS ARE IN ORDER
-
   ordered.data <- ordered.data[order(ordered.data[,id.var], ordered.data[,t.var]), ]
-  compmat <- data.table::dcast(data.table::as.data.table(ordered.data), formula = paste0(id.var, "~", t.var), value.var = treatment.var)
-  ts <- as.numeric(unlist(strsplit(names(matched_sets), split = "[.]"))[c(F,T)])
-  tids <- as.numeric(unlist(strsplit(names(matched_sets), split = "[.]"))[c(T,F)])
+  compmat <- data.table::dcast(data.table::as.data.table(ordered.data),
+                               formula = paste0(id.var, "~", t.var), value.var = treatment.var)
+  ts <- as.numeric(sub(".*\\.", "", names(matched_sets)))
+  tids <- as.numeric(sub("\\..*", "", names(matched_sets)))
   class(matched_sets) <- "list" #so that Rcpp::List is accurate when we pass it into cpp functions
   compmat <- data.matrix(compmat)
 
   idx <- check_treated_units_for_treatment_reversion(compmat = compmat, compmat_row_units = as.numeric(compmat[, 1]),
                                                      compmat_cols = as.numeric(colnames(compmat)[2:ncol(compmat)]),
                                                      lead = max.lead, treated_ids = tids, treated_ts = ts)
-  if(all(!idx)) stop("estimation not possible: All treated units are missing data necessary for the calculations to proceed")
-  if(any(!idx))
+  if (all(!idx)) stop("estimation not possible: All treated units are missing data necessary for the calculations to proceed")
+  if (any(!idx))
   {
     class(matched_sets) <- c("matched.set", "list") #to get the matched.set subsetting with attributes
     matched_sets <- matched_sets[idx]
@@ -680,7 +933,7 @@ enforce_lead_restrictions <- function(matched_sets, ordered.data, max.lead, t.va
   #probably should rename this function, but working in a similar context here so seeing if it works
   idx <- needs_renormalization(ll)
   class(matched_sets) <- c("matched.set", "list")
-  if(any(idx))
+  if (any(idx))
   {
     sub.index <- ll[idx]
     sub.set <- matched_sets[idx]
@@ -691,7 +944,7 @@ enforce_lead_restrictions <- function(matched_sets, ordered.data, max.lead, t.va
     sub.set.new <- mapply(FUN = create_new_sets, sub.set, sub.index, SIMPLIFY = FALSE)
     attributes(sub.set.new) <- attributes(sub.set)
     all.gone.counter <- sapply(sub.set.new, function(x){sum(x)})
-    if(sum(all.gone.counter == 0) > 0) #case in which all the controls in a particular group were dropped
+    if (sum(all.gone.counter == 0) > 0) #case in which all the controls in a particular group were dropped
     {
       #warning("all controls in a particular matched set were removed due to missing data")
       #browser()
@@ -705,7 +958,7 @@ enforce_lead_restrictions <- function(matched_sets, ordered.data, max.lead, t.va
       sub.set.new <- mapply(FUN = create_new_sets, sub.set, sub.index, SIMPLIFY = FALSE)
       attributes(sub.set.new) <- attributes(sub.set)
     }
-    if(all(sapply(sub.set.new, length) == 0)) stop('estimation not possible: none of the matched sets have viable control units due to a lack of necessary data')
+    if (all(sapply(sub.set.new, length) == 0)) stop('estimation not possible: none of the matched sets have viable control units due to a lack of necessary data')
     #pm2 <- perform_refinement(ordered.data = ordered.data, mset.object = sub.set.new)
     #matched_sets[idx] <- pm2
     matched_sets[idx] <- sub.set.new
@@ -718,9 +971,9 @@ enforce_lead_restrictions <- function(matched_sets, ordered.data, max.lead, t.va
 gather_msm_sets <- function(lead.data.list)
 {
   number.of.sets <- sapply(lead.data.list, length)
-  if(length(unique(number.of.sets)) != 1) stop("error with matched sets in msm calculations")
+  if (length(unique(number.of.sets)) != 1) stop("error with matched sets in msm calculations")
   number.of.sets <- unique(number.of.sets)
-  long.data.lead.list <- unlist(lead.data.list, recursive = F)
+  long.data.lead.list <- unlist(lead.data.list, recursive = FALSE)
 
   long.weights.list <- lapply(long.data.lead.list, function(x){return(as.vector(x[, 4]))})
   multiplied.weights <-  multiply_weights_msm(long.weights.list, number.of.sets)
@@ -730,8 +983,29 @@ gather_msm_sets <- function(lead.data.list)
     set[, "ps"] <- weights #again this ps is misleading but for consistency with the other functions lets go with it
     return(set)
   }
-  reassembled.sets <- mapply(FUN = reassemble.weights, set = reassembled.sets, weights = multiplied.weights, SIMPLIFY = F)
+  reassembled.sets <- mapply(FUN = reassemble.weights, set = reassembled.sets,
+                             weights = multiplied.weights, SIMPLIFY = FALSE)
 
   return(reassembled.sets)
 }
 
+
+merge_formula <- function(form1, form2)
+{
+
+  rhs1 <- trimws(unlist(strsplit(as.character(form1)[2], "\\+")))
+  rhs2 <- strsplit(deparse(form2[[2]]), " \\+ ")[[1]]
+
+  # create the merged rhs and lhs in character string form
+  rhs <- c(rhs1, rhs2)
+
+  # put the two sides together with the amazing
+  # reformulate function
+  out <- reformulate(rhs)
+
+  # set the environment of the formula (i.e. where should
+  # R look for variables when data aren't specified?)
+  environment(out) <- environment(form1)
+
+  return(out)
+}
